grammar Assembler ;

file : line+;

line : instruction
     | label_decl
     | mem_assign
     | COMMENT
     | line COMMENT
     | line SPACE COMMENT
     | line SPACE
     | line '\r'? '\n'
     | '\r'? '\n'
     ;

instruction : iname SPACE icode
            | iname SPACE label
            | iname SPACE mem_const
            | iname
            ;

label : LABEL;
mem_const : MEM_CONST;
icode : ICODE;
iname : INAME;
label_decl : LABEL_DECL;
mem_assign  : MEM_CONST SPACE ICODE;


// Fragements
fragment BIT        : '0' .. '1';
fragment DIGIT      : '0' .. '9';
fragment UPPER      : 'A' .. 'Z';
fragment LOWER      : 'a' .. 'z';
fragment HEX_UPPER  : 'A' .. 'F';
fragment HEX_LOWER  : 'a' .. 'f';
fragment HEX_LETTER : HEX_LOWER | HEX_UPPER;
fragment HEX_NUM    : HEX_LETTER | DIGIT;
fragment LETTER     : LOWER | UPPER;
fragment WORD       : LETTER | '_';
fragment ALPHANUM   : WORD | DIGIT;
fragment EMPTY_SPACE      : ' ' | '\t';

// Tokens
COMMENT    : '#' .*? ('\r'? '\n' | EOF);
LABEL_DECL : ALPHANUM+ ':';
LABEL      : ':' ALPHANUM+;
MEM_CONST  : '$' + ALPHANUM+;
INAME      : LETTER ALPHANUM*;
ICODE      : ('-'? DIGIT+) | ('-'? '0x' HEX_NUM+) | ('-'? '0b' BIT+) | ('-'? '0X' HEX_NUM+) | ('-'? '0B' BIT+);
SPACE      : EMPTY_SPACE+;

WS         : [\n\r]+ -> skip ;