import org.antlr.v4.runtime.ANTLRInputStream;
import org.antlr.v4.runtime.CommonTokenStream;
import org.antlr.v4.runtime.ParserRuleContext;
import org.antlr.v4.runtime.tree.ParseTree;
import org.antlr.v4.runtime.tree.ParseTreeWalker;

import java.io.IOException;
import java.io.InputStream;

/**
 * @author Karl Janson
 * @since 11/23/14
 * Class for reading and analyzing the Assembler file.
 */
public class AsmReader {

    /**
     * Parses the assembler file
     * @param is inputstream from the assembler file
     * @param showTree boolean to show the syntax tree
     * @return Loader instance for AsmLoader
     * @throws Exception
     */
    public AsmLoader readAsm(InputStream is, boolean showTree) throws Exception {
        AssemblerLexer lexer;

        try {
            lexer = new AssemblerLexer(new ANTLRInputStream(is));
        } catch (IOException e) {
            System.out.println("Error opening intruction codes file");
            return null;
        }
        CommonTokenStream tokens = new CommonTokenStream(lexer);
        AssemblerParser parser = new AssemblerParser(tokens);

        if (showTree) {

            /* show parse tree in gui */
            ParserRuleContext ruleContext = parser.file();
            ruleContext.inspect(parser);
            return null;

        } else {
            parser.setBuildParseTree(true); // tell ANTLR to build a parse tree
            ParseTree tree = parser.file();

            /* walk the tree */
            ParseTreeWalker walker = new ParseTreeWalker();
            AsmLoader loader = new AsmLoader();
            try {
                walker.walk(loader, tree);
            } catch (Exception e) {
                System.out.println(e.getMessage());
                return null;
            }

            if (loader.getExceptionOccurred()) {
                return null;
            } else {
                return loader;
            }
        }
    }
}
