import org.antlr.v4.runtime.ANTLRInputStream;
import org.antlr.v4.runtime.CommonTokenStream;
import org.antlr.v4.runtime.ParserRuleContext;
import org.antlr.v4.runtime.tree.ParseTree;
import org.antlr.v4.runtime.tree.ParseTreeWalker;

import java.io.IOException;
import java.io.InputStream;
import java.util.Map;

/**
 * Class for reading and analyzing the instructions file.
 * @author Karl Janson
 * @since 11/22/14
 */
public class InstrReader {

    /**
     * Class for reading the instructions.
     * @param is inputstream from instructions file
     * @param showTree boolean to show the syntax tree (debug option)
     * @return Map of instructions
     * @throws Exception
     */

    public Map<String, String> readInstr(InputStream is, boolean showTree) throws Exception {
        OpcodesLexer lexer;

        try {
            lexer = new OpcodesLexer(new ANTLRInputStream(is));
        } catch (IOException e) {
            System.out.println("Error opening intruction codes file");
            return null;

        }
        CommonTokenStream tokens = new CommonTokenStream(lexer);
        OpcodesParser parser = new OpcodesParser(tokens);

        if (showTree) {
                /* show parse tree in gui */
            ParserRuleContext ruleContext = parser.file();
            ruleContext.inspect(parser);
            return null;

        } else {
            parser.setBuildParseTree(true); // tell ANTLR to build a parse tree
            ParseTree tree = parser.file();

            /* walk the tree */
            ParseTreeWalker walker = new ParseTreeWalker();
            InstrLoader loader = new InstrLoader();
            try {
                walker.walk(loader, tree);
            } catch (Exception e) {
                System.out.println(e.getMessage());
                return null;
            }

            return loader.getInstructionList();
        }
    }
}
